import os
from dataclasses import dataclass


IMGS_PATH = os.path.join(os.path.dirname(__file__), "imgs")

THOUGHT = os.path.join(IMGS_PATH, "thought.png")
WHAT = os.path.join(IMGS_PATH, "what.png")
ZEN = os.path.join(IMGS_PATH, "zen.png")
NO = os.path.join(IMGS_PATH, "no.png")


@dataclass
class Screen:
    text: str
    bg_path: str
    font: str = "Helvetica 16"

    def __hash__(self):
        return hash(self.text)


@dataclass
class Option:
    text: str
    next: Screen


start = Screen(text="Oi, what do you want?", bg_path=WHAT)
idea = Screen(text="Well, what do you have in mind?", bg_path=THOUGHT)
idea_r = Screen(text="NO.\nThat's a terrible idea.\nThink about it some\n more.", bg_path=NO, font="Helvetica 18 bold")
can_we = Screen(text="Hmmm so what do we need to do?", bg_path=THOUGHT)
kai = Screen(text="Are you Kai?", bg_path=WHAT)
kai_y = Screen(text="NO. We can't do that.", bg_path=NO, font="Helvetica 18 bold")
kai_n = Screen(text="Well, everything is a trade off.\nConsider first what is required functionally - meaning, "
                    "what does the thing NEED to do at all, then consider performance requirements.\nQuestions to ask yourself-\n"
                    " - Where are the bottlenecks?\n"
                    " - What is a reasonable load? What increases it? (more cameras? users? msgs?)\n"
                    " - Where can load be distributed & handling parallelized? (more services of some type? servers?)\n"
                    " - Where can requirements be relaxed? (example, in History-saving requirement of latency is relaxed)\n"
                    "Remember that no system can do all the load with all the accuracy and zero latency.\n"
                    "The important part of being a SW engineer is finding which trade-offs are relevant to your case right now.\n"
                    "I'm afraid ChatGPT won't help in many cases, BUT most problems are already \"solved problems\" - "
                    "someone already had to deal with them - so it's always wise to see what other people did in similar cases and learn.\n"
                    "Best of luck!", bg_path=ZEN, font="Helvetica 13")
back_option = Option(text="Ask something else", next=start)

BYE_TXT = "Nevermind, bye!"

StateM = {
    start: [Option(text="Say, how can we do the thing?", next=can_we),
            Option(text="I have an idea I wanna discuss", next=idea)],
    idea: [Option(text="Tell me about your idea\nclick here when done", next=idea_r)],
    idea_r: [back_option],
    can_we: [Option(text="Tell me what we need to do\nclick here when done", next=kai)],
    kai: [Option(text="yes", next=kai_y), Option(text="no", next=kai_n)],
    kai_y: [back_option],
    kai_n: [back_option]
}



from kerenor.states import StateM, Screen, start, BYE_TXT

try:
    import tkinter as tk
    has_tk_inter = True
except ModuleNotFoundError:
    has_tk_inter = False

root = None
canvas = None
bg_img = None


def user_input(num_options: int):
    userinput = -1
    while userinput < 0 or userinput > num_options-1:
        userinput = input("press number!\n")
        try:
            userinput = int(userinput)
        except:
            userinput = num_options + 1
    return userinput


def make_button(text: str, y_loc: int, canvas, next_option):
    double_line = text.find("\n")
    len_line = (len(text) if double_line < -1 else max(double_line, len(text) - double_line))
    button = tk.Button(root, text=text, command=next_option,
                       anchor='sw', width=len_line, background="#33B5E5", activebackground="#33C5DD")
    canvas.create_window(10, y_loc, anchor='nw', window=button)
    y_loc += 60 if double_line > 0 else 40
    return y_loc


def draw_wrapper(state: Screen):
    def draw_this_state():
        return draw_state_on_canvas(state)
    return draw_this_state


def draw_state_on_canvas(state: Screen):
    global root
    global canvas
    global bg_img
    if not canvas:
        canvas = tk.Canvas(root, width=520, height=520, bg='white')
    else:
        canvas.delete("all")
    text_note = tk.Label(canvas, text=state.text, font=state.font, wraplength=500, justify="left", bg='white')
    canvas.create_window(10, 20, window=text_note, anchor='nw')
    bg_img = tk.PhotoImage(file=state.bg_path)
    canvas.create_image(260, 260, image=bg_img)

    # buttons!
    y_loc = 470 - len(StateM[state]) * 50
    for ind, option in enumerate(StateM[state]):
        y_loc = make_button(text=option.text, canvas=canvas, y_loc=y_loc, next_option=draw_wrapper(option.next))
    make_button(text=BYE_TXT, canvas=canvas, y_loc=y_loc, next_option=root.quit)
    canvas.pack()


def run_tk():
    global root
    root = tk.Tk()
    root.title('KerenOr')
    draw_state_on_canvas(start)
    root.mainloop()


def eval_state(state: Screen):
    print(state.text)
    exit_ind = len(StateM[state]) + 1
    for option_ind, option in enumerate(StateM[state]):
        print(f"{option_ind + 1}.\t{option.text.replace('click here', 'select number')}")
    print(f"{exit_ind}.\t{BYE_TXT}")
    userinput = user_input(exit_ind + 1)
    if userinput == exit_ind:
        print("Bye!")
        exit(0)

    return StateM[state][userinput - 1].next


def run_cmd():
    current_state = start
    while True:
        current_state = eval_state(current_state)


def run_me():
    if has_tk_inter:
        run_tk()
    else:
        run_cmd()




from setuptools import setup, find_namespace_packages

PROJECT = "kerenor"

setup(
    name=PROJECT,
    version='0.1.0',
    description=PROJECT,
    author=PROJECT,
    package_data={PROJECT: ["./imgs/*"]},
    include_package_data=True,
    packages=find_namespace_packages(include=[f"{PROJECT}*"])
)
